#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Basic DB2 instance heartbeat monitor for non-DBA staff.
# Report on every DB2 instance at the server.
# Run through root only at each database server in scope.
# Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: <path>/DB2Checks.ksh without any arguments
#
# Maintenance:
# 1. This is not a replacement for proper monitoring, just an ad hoc and very basic stopgap.
# 2. Use SMTP for alert notifications, once it is enabled.
# 3. A list of DB2 instance and database names by server is at:
# https://confluence.dev.bbc.co.uk/display/DMI/EM3+DB2+Database+inventory?showChildren=true#children
# 4. Edit $db2_install_path when we to the next release of DB2 - do not edit for V9.7 fixpacks.
# Alex Levy, 23.02.2012

function redirect_output {
# Function to re-direct stdin/stderr to log file.
 exec 2>&1
 exec 1>> $log
}

function reset_output {
# Restore output back to stdin and stderr.
# Counterpart to redirect_output
 exec 1>&-
 exec 1>&2
}

function find_smtp {
# Ping the smtp gateway to see if it is accessible.
# If so, script will mail results.
typeset -i rc=0
ping -c 1 -w 30 smtp | grep received > /tmp/${job}.ping.out
rc=$?
if   (( rc > 0 ))
then smtp_on=0
else smtp_on=$(awk '{print $4}' /tmp/${job}.ping.out)
fi
(( smtp_on == 0 )) && print "SMTP Gateway is not available."
}

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1
typeset BOLD=$(tput smso)
typeset PLAIN=$(tput rmso)
(( $# != 2 )) && print "\n${BOLD}ERROR: $0 expects 2 arguments but got $# ${PLAIN}\n" && exit 99
print ${BOLD}${2}${PLAIN}
db2 +c -o- connect # Is there a current connection to the database?
(( $? == 0 )) && db2 -v rollback # abort any open transactions
reset_output
(( smtp_on == 0 )) && cat $log
(( smtp_on == 1 )) && cat $log | mail -s "$job output from server $here" Alex.Levy@bbc.co.uk -c Mark.Povah@bbc.co.uk
rm -f /tmp/${job}*
exit $1
}

function dbcheck {
# run some very basic checks for each database in active instance $1
# 1.uptime
# 2.db memory compared to server memory
# 3 db activated
# 4.Connectivity check
# Done by Nagios: 5.Include server or O/S checks for missed Simpana backups.
# Done by Nagios: 6.DB2 filesystem utilisation
# 7.Test for connections > n% max_connections.
typeset -l instance=$1
typeset -i dbmem_pct_used    # % current partition memory / max partition memory
typeset -i appls maxappls    # actual and maximum number of database connections 
typeset -L10 autoconn=NONE   # is db cfg MAXAPPLS automatic? values NONE or AUTOMATIC
typeset -i conn_pct          # % appls / maxappls
typeset -i rc=0 count=0 

#generate database list
cat > /tmp/${job}.generate_${instance}_dblist << END
#!/bin/ksh
job=$job
. ~${instance}/sqllib/db2profile 
db2 list db directory | egrep "Database name" | awk -F"= " '{print NR,\$2}' > /tmp/\${job}.dbnames
db2 list db directory | egrep "entry type"    | awk -F"= " '{print NR,\$2}' > /tmp/\${job}.dbtypes
join -o 1.2 2.2 /tmp/\${job}.dbnames /tmp/\${job}.dbtypes > /tmp/\${job}.dblist
sort -u /tmp/\${job}.dblist > /tmp/\${job}.dblist.sort
if   [[ -s /tmp/\${job}.dblist.sort ]]
then # file is populated, do nothing yet 
     :
else leave 1 "There are no active databases for instance ${instance}."
fi
END

chmod +x /tmp/${job}.generate_${instance}_dblist
su - $instance -c /tmp/${job}.generate_${instance}_dblist

cat > /tmp/${job}.checks << END
#!/bin/ksh
. ~${instance}/sqllib/db2profile 

function report_error {
# Returns to caller with message $2 and caller-supplied return-code $1
# Caller then returns to main program with that code. Have to do this since this part of the script is built dynamically and cat'd out for root execution.
typeset BOLD=$(tput smso)
typeset PLAIN=$(tput rmso)
(( $# != 2 )) && print "\n${BOLD}ERROR: $0 expects 2 arguments but got $# ${PLAIN}\n" && exit 99
print ${BOLD}${2}${PLAIN}
db2 +c -o- connect # Is there a current connection to the database?
(( $? == 0 )) && db2 -v rollback # abort any open transactions
return $1
}

db2 -v attach to ${instance}  
rc=$? ; (( rc > 0 )) && report_error $rc "Attachment failure for instance ${instance}." 

print "These memory figures are not evaluated here but supplied for additional diagnostics if an error is flagged."
#report uptime for instance
db2pd -memsets | awk -F "--" '{if (NF == 4) print \$3,"at",\$4}'
# report server memory and swap space
print Server memory \$(top | head | grep Mem:)
top | head | grep -i Swap
#instance memory summary
db2mtrk -i -v
#database memory summary (all local dbs in the instance)
db2mtrk -d -v | egrep -v "Application Heap"

# Ignore remote databases, since they will be monitored at the remote server;
# the following code therefore works for client instances too :-)
while read name type
do
    if [[ \$type == "Indirect" ]]
    then
         count=\$(db2 list active databases | grep -c \$name)
         (( count == 1 )) && print Database \$name is activated
         (( count == 0 )) && report_error 1 "Database \$name was not activated"
         db2 -v connect to \$name
         rc=\$? ; (( rc > 0 )) && report_error $rc "Connection failure for instance ${instance} database \${name}."
         db2 -v "select substr(name,1,20) as name, substr(value,1,10) as value from sysibmadm.env_sys_resources where name like '%MEM%'"
         # INSTANCE_MEMORY is usually AUTOMATIC so inspect current/peak v. current/max memory usage, since per partition maximum can grow.
         dbmem_pct_used=\$(db2 -x "select smallint(decimal(decimal(current_partition_mem)/ decimal(peak_partition_mem),8,5) * 100) from table (sysproc.admin_get_dbp_mem_usage(-1)) as t")
         rc=\$? ; (( rc > 0 )) && report_error $rc "Fatal error database \$name querying table function sysproc.admin_get_dbp_mem_usage."
         # set threshold high at 95% because we use STMM and some memory is not swapped out
         (( dbmem_pct_used > 95 )) && report_error 99 "Database \${name} current partition memory utilisation of \${dbmem_pct_used}\% exceeds 95% of peak memory for the database partition."
         print "Database \${name}: current partition memory utilisation is \${dbmem_pct_used}%."
         # If MAXAPPLS is AUTOMATIC, don't report a high level of current connections.
         autoconn=\$(db2 -x "select value_flags from sysibmadm.dbcfg where name = 'maxappls'")
         rc=\$? ; (( rc > 0 )) && report_error $rc "Fatal error database \$name querying admin view sysibmadm.dbcfg."
         if   [[ autoconn == "AUTOMATIC " ]]
         then maxappls=\$(db2 -x "select value from sysibmadm.dbcfg where name = 'maxappls'")
              rc=\$? ; (( rc > 0 )) && report_error \$rc "Fatal error database \$name querying admin view sysibmadm.dbcfg."
              appls=\$(db2 -x "select appls_cur_cons from sysibmadm.snapdb")
              rc=\$? ; (( rc > 0 )) && report_error \$rc "Fatal error database \$name querying admin view sysibmadm.snapdb."
              conn_pct=appls*100/maxappls
              (( conn_pct > 90 )) && report_error 99 "database \${name}: current connections \$appls is now \${conn_pct}% of maximum allowed \${maxappls}."
         fi     
         db2 -v connect reset
    fi     
done < /tmp/${job}.dblist.sort
END
chmod 555 /tmp/${job}.checks
su - $instance -c /tmp/${job}.checks
rc=$?
return $rc
}

# Main
set -u        # assert undefined variables are not to be used
typeset BOLD=$(tput smso)
typeset PLAIN=$(tput rmso)
typeset here=$(hostname)
typeset timestamp=$(date +%Y%m%d%H%M)
typeset job=$(basename $0 .ksh)
typeset log=/tmp/${job}.${timestamp}
typeset db2_install_path=/opt/ibm/db2/V9.7 # will change at next version
typeset db2_cmd_path=
typeset -l instance=
typeset -l instance_state=
typeset -i smtp_on=0 # 0 = no working SMTP gateway, 1 = we can use SMTP 
typeset -i count=0
typeset -i exitcode=0
typeset -i retcode=0
typeset me=$(whoami)

#redirect_output
find_smtp

# Check root is running this - the db2gcf command requires either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then :
else
     leave 99 "$0 : This program is only designed to run from root. Sorry."
fi

# Check if this is a DB2 server - is DB2 installed?
[[ -d /opt/ibm/db2/V9.7 ]] || leave 0 "$0 : DB2 is not installed here at ${here}. Nothing to check. Exiting."
# Check the db2gcf and db2ilist commands are executable
[[ -f /opt/ibm/db2/V9.7/bin/db2gcf && -x /opt/ibm/db2/V9.7/bin/db2gcf ]] || leave 99 "$0 : Cannot execute /opt/ibm/db2/V9.7/bin/db2gcf as ${me}. Check DB2 installation."
[[ -f /opt/ibm/db2/V9.7/instance/db2ilist && -x /opt/ibm/db2/V9.7/instance/db2ilist ]] || leave 99 "$0 : Cannot execute /opt/ibm/db2/V9.7/instance/db2ilist as ${me}.Check DB2 installation."

# Loop through a list of instances from the db2ilist command.
print "$0 : DB2 instances at server ${here} are: "
/opt/ibm/db2/V9.7/instance/db2ilist | tee /tmp/${job}.db2ilist
# For each instance, check if the instance has failed over; the tell-tale sign is if key directories such as sqllib are still under $INSTHOME.or have been remounted to the standby server.
# Further, test the path to DB2 commands must be executable.
# If successful, execute the db2profile for each instance in the current shell (to run DB2 commands later)
for instance in $(cat /tmp/${job}.db2ilist)
do
    db2_cmd_path=~${instance}/sqllib/bin
    if   [[ -d ${db2_cmd_path} && -x ${db2_cmd_path} && -x ~${instance}/sqllib/db2profile ]]
    then . ~${instance}/sqllib/db2profile
         instance_state=$(/opt/ibm/db2/V9.7/bin/db2gcf -s -i $instance | egrep "DB2 State" | awk -F": " '{print $2}')
         case $instance_state in
              "available") print "DB2 instance $instance is up.\n Running some basic checks."
                           dbcheck $instance
                           retcode=$? ; (( exitcode += retcode ))
                           ;;
              "operable" ) print "DB2 instance $instance is down but can be brought up."
                           let exitcode=exitcode+1
                           ;;
              "not ope*" ) leave 99 "DB2 instance $instance is not operable and cannot be brought up."
                           ;;
         esac
    else
         print ${BOLD} "$0 : It looks like DB2 instance $instance has failed over." ${PLAIN}
         let exitcode=exitcode+1
    fi
done

(( exitcode == 0 )) && leave 0 "DB2 appears healthy at ${here}." 
(( exitcode > 0  )) && leave $exitcode "Please review warning messages above. Seek DBA assistance if necessary." 
