#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins, Confluence page: https://repo.dev.bbc.co.uk/dmi/em3/dev/trunk/EM3_DB/core/deploy_db/bin/NAGIOS/ 
# Script:		DB2_Database_Test12.ksh
# Arguments:	$1 instance name
#				$2 database alias 
# Synopsis:		Health check for Sorts per Transaction (SRTTX) metric. The lower the SRTTX, the better.
#				Adjust warning and danger thresholds as required.				
#				SRTTX metric within safety limit 0-1	return code 0
#				SRTTX within warning zone of 1-2     	return code 1
#				SRTTX in danger zone of 2+  			return code 2 
#				Any other error     					return code 3
#
# Instructions: #12 of several very basic DB2 monitors for non-DBA staff monitoring through Nagios.
#				Do NOT run if an earlier test has failed - we assume at this stage that the instance is up and the database is activated and connectable.
# 				Run through root only at each database server in scope. 
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Database_Test12.ksh instancename dbname
#
# DB2 Notes:	1. The warning and danger thresholds are initially somewhat arbitrary - we know the production metric of 978 on 25/03/2013 is very bad.
#
# Maintenance:
# 1. This is not a replacement for a proper monitoring soluion like DBI tools, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. Code contains a semaphore check in the form of a lock file to ensure Nagios doesn't run two or more instances of this script in parallel.
# 	 The semaphore check will exit immediately if a lock file is in place and another instance of the same job is running. 
#	 Do not remove the lockfile under this condition.
#
# Alex Levy, t/a http://www.sustainablesoftware.net March 2013
#
# Amendment History
# -----------------
#
#
###########################################

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 != 4 ]] 
then rm -f $lockfile
fi
exit $RC
}

# Main
set -u        		# assert variables must be defined
typeset job=$(basename $0 .ksh)
typeset -i count=0	# counter for leave function
typeset here=$(hostname)
typeset me=$(whoami)
(( $# == 2 )) || leave 3 "$job expects 2 arguments but received $# - usage $0 DB2instancename databasename."
typeset -l instance=$1
typeset -u dbname=$2
typeset -i rc=0		# *nix return code 
typeset lockfile=defined_further_down
typeset -i RC 		# set in function leave
typeset -i jobcount=0
typeset srttx		# set below

# Check root is running this - some commands below require either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then : 
else
     leave 1 "$0 : This program is only designed to run from root. Sorry."
fi

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${dbname}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | grep $dbname | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# ... and reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
touch $lockfile

# Generate a script for the instance owner:
# The computed metric from the called SQL script has 2 decimal places.
# This will exit with a return code = SRTTX * 100, or 999999999 (an impossible value) on a SQL error
cat > /tmp/${job}.SRTTX.${dbname} << END
#!/bin/ksh

typeset -F2 SRTTX_2DP=0.00 SRTTX_0D=0 # threshold counters to 2 decimal places.
typeset -i SRTTX=0 		
typeset -i rc=0 
. ~${instance}/sqllib/db2profile

db2 -v "connect to $dbname" 
rc=\$?

if (( rc == 0 ))
then
    SRTTX_2DP=\$(db2 -xtf ~$instance/deploy_db/bin/NAGIOS/HEALTH/SRTTX.sql)
	rc=\$? 
fi

db2 -v terminate
print "$(date) return code \$rc SRTTX value \$SRTTX" 
# set final exit code (see comments and root's case statement below for interpretation ))
if   (( rc == 0 ))
then print \$SRTTX_2DP
	 ((SRTTX_0DP = SRTTX_2DP * 100 )) # zeroise the fractional part
	 SRTTX=\$SRTTX_0DP				  # cast to integer
	 exit \$SRTTX
else # signal a SQL or connection error
     exit 999999999
fi
END

chmod +x  /tmp/${job}.SRTTX.${dbname} 
trap " " ERR # turn trap off as we want to evaluate the return code explicitly:
su - $instance -c "/tmp/${job}.SRTTX.${dbname} > /tmp/${job}.SRTTX.${dbname}.out " 
rc=$? 
(( rc == 999999999 )) && print "$0 : SQL query or connection error - please review /tmp/${job}.SRTTX.${dbname}.out."
(( rc == 999999999 )) && exit 3
srttx=$(cat /tmp/${job}.SRTTX.${dbname}.out | tail -1)
(( rc <= 100 )) && print "$0 : SRTTX metric of $srttx is within acceptable limits at database ${dbname}."
(( rc <= 100 )) && exit 0
(( rc <= 200 )) && print "$0 : SRTTX metric of $srttx exceeds warning threshold at database ${dbname}."
(( rc <= 200 )) && exit 1
(( rc > 200 ))  && print "$0 : SRTTX metric of $srttx exceeds danger threshold at database ${dbname}."
(( rc > 200 ))  && exit 2
