#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins.
# Script:		DB2_Database_Test2.ksh
# Arguments:	$1 instance name
#				$2 database name or alias 
# Synopsis:		Is DB2 database activated? yes 	return code 0
#				No, but have activated it here 	return code 1
#				No, cannot activate            	return code 2
#				Any other error     			return code 3
#
# Instructions: #2 of several very basic DB2 heartbeat monitors for non-DBA staff monitoring through Nagios.
#				Do NOT run if an earlier test has failed - we assume at this stage that the instance is activated.
# 				Run through root only at each database server in scope.
# 				Run for each DB2 instance and database at the server.
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Database_Test2.ksh instancename dbname
#
# Maintenance:
# 1. This is not a replacement for proper monitoring, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. Code contains a semaphore check in the form of a lock file to ensure Nagios doesn't run two or more instances of this script in parallel.
# 	 The semaphore check will exit immediately if a lock file is in place and another instance of the same job is running. 
#	 Do not remove the lockfile under this condition.
#
# Alex Levy, t/a http://www.sustainablesoftware.net October 2012
#
#------------------------------------------

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 -ne 4 ]] 
then rm -f $lockfile
fi
exit $RC
}

set -u        # assert undefined variables are not to be used
typeset here=$(hostname)
typeset timestamp=$(date +%Y%m%d%H%M)
typeset job=$(basename $0)
typeset db2_install_path=     	# DB2 V9.7 value is /opt/ibm/db2/V9.7 on Linux, but changes at each version/release 
typeset me=$(whoami)
typeset -i rc=0					# *nix return code 
typeset -i listed=0 activated=0 # counters
typeset -u dbtarget				# resolve supplied alias name to 'real' database name
typeset -l instance				# set later
typeset lockfile=defined_further_down
typeset -i RC 				# set in function leave
typeset -i jobcount=0

# Check root is running this - some commands below require either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then : 
else
	 leave 1 "$0 : This program is only designed to run from root. Sorry."
fi

(( $# == 2 )) || leave 3 "$job expects 2 arguments but received $# - usage $0 DB2instancename databasename."
typeset -l instance=$1
typeset -u dbname=$2

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${dbname}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | grep $dbname | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# ... and reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
touch $lockfile

# Remove guff from the last run - name patterns must exclude the lockfile.
rm -f /tmp/${job}/db*.${2} /tmp/${job}.generate_${instance}_dblist.${dbname} > /dev/null

# Generate a database list for this instance, mapping aliases to the underlying database names and testing whether the database is local or remote;
# - commands like list active databases only display the db name, never the alias; utilities like db2pd do not accept alias names;
# - here, we're only interested in local databases (entry type = 'Indirect'), not remotely catalogued databases for other instances.
cat > /tmp/${job}.generate_${instance}_dblist.${dbname} << END
#!/bin/ksh
job=$job
. ~${instance}/sqllib/db2profile
db2 list db directory | egrep "Database name"  | awk -F"= " '{print NR,\$2}' > /tmp/\${job}.dbnames.${dbname}		# sequence dbname
db2 list db directory | egrep "Database alias" | awk -F"= " '{print NR,\$2}' > /tmp/\${job}.dbaliases.${dbname}		# sequence alias	
db2 list db directory | egrep "entry type"     | awk -F"= " '{print NR,\$2}' > /tmp/\${job}.dbtypes.${dbname}		# sequence Indirect/Remote
join -o 1.1 1.2 2.2  /tmp/\${job}.dbnames.$dbname /tmp/\${job}.dbtypes.$dbname > /tmp/\${job}.dblist.${dbname}		# dbname   Indirect/Remote
join -o 1.1 1.2 2.2  /tmp/\${job}.dbaliases.$dbname  /tmp/\${job}.dbtypes.$dbname > /tmp/\${job}.dblist2.${dbname}	# alias	   Indirect/Remote
join -o 1.2 2.2 1.3  /tmp/\${job}.dblist.$dbname /tmp/\${job}.dblist2.$dbname > /tmp/\${job}.dblist3.${dbname}		# alias dbname Indirect/Remote
sort -u /tmp/\${job}.dblist3.${dbname} > /tmp/\${job}.dblist.sort.${dbname}
if   [[ -s /tmp/\${job}.dblist.sort.${dbname} ]]
then # there is at least 1 catalogued database, either local or remote.
	 db2 list active databases > /tmp/\${job}.active_dbs.${dbname}
     return 0
else return 1
fi
END

chmod +x /tmp/${job}.generate_${instance}_dblist.${dbname}
su - $instance -c /tmp/${job}.generate_${instance}_dblist.${dbname} || leave 1 "$0 : Instance $instance has no catalogued databases - check output under /tmp/{job}.xxx.${dbname}\*"

while read name alias type
do
	if  [[ $type == "Indirect" ]]
	then
		if   [[ $alias == $dbname && $name == $dbname ]]
		then (( listed +=1 ))
			 dbtarget=$name
		break
		fi
		if  [[ $alias == $dbname ]]
		then (( listed +=1 ))
			 dbtarget=$name
			 break
		fi
	fi
done < /tmp/${job}.dblist.sort.${dbname}

(( listed == 0 )) && leave 1 "$dbname is not catalogued as a local database  at instance $instance ; please check call $0 $*"
trap " " ERR # turn trap off for positive grep and db2 command line return codes
activated=$(grep -c $dbtarget  /tmp/${job}.active_dbs.${dbname}) || :
(( activated == 1 )) && leave 0 "$0 : database $dbname is activated at instance ${instance}."
if  (( activated == 0 ))
then cat > /tmp/${job}_activate_${dbtarget}.${dbname} << END
	 activate db $dbtarget
END
     su - ${instance} -c "db2 -vf /tmp/${job}_activate_${dbtarget}.${dbname} > /tmp/${job}_activate_${dbtarget}.${dbname}.out"
	 rc=$?
	 (( rc == 0 )) &&  leave 1 "$0 : database $dbname was found deactivated, but was reactivated at instance ${instance}."
	 (( rc > 0 ))  &&  leave 2 "$0 : database $dbname could not be reactivated at instance ${instance}."
fi
