#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins.
# Script:		DB2_Database_Test3.ksh
# Arguments:	$1 instance name
#				$2 database name or alias 
# Synopsis:		Is database connectable?   yes, return code 0
#				No, cannot cannot connect      	return code 2
#				Any other error     			return code 3
#
# Instructions: #3 of several very basic DB2 heartbeat monitors for non-DBA staff monitoring through Nagios.
#				Do NOT run if an earlier test has failed - we assume at this stage that the instance is up and the database is connectable. 
# 				Run through root only at each database server in scope.
# 				Run for each DB2 instance and database at the server.
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Database_Test3.ksh instancename dbname
#
# Maintenance:
# 1. This is not a replacement for proper monitoring, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. Code contains a semaphore check in the form of a lock file to ensure Nagios doesn't run two or more instances of this script in parallel.
# 	 The semaphore check will exit immediately if a lock file is in place and another instance of the same job is running. 
#	 Do not remove the lockfile under this condition.
#
# Alex Levy, t/a http://www.sustainablesoftware.net October 2012
#
###########################################

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 -ne 4 ]] 
then rm -f $lockfile
fi
exit $RC
}

# Main
set -u        	# assert undefined variables are not to be used
typeset job=$(basename $0)
typeset me=$(whoami)
typeset -i rc=0	# *nix return code 
typeset lockfile=defined_further_down
typeset -i RC 				# set in function leave
typeset -i jobcount=0

(( $# == 2 )) || leave 3 "$job expects 2 arguments but received $# - usage $0 DB2instancename databasename."
typeset -l instance=$1
typeset -u dbname=$2

# Check root is running this - some commands below require either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then  
	 # remove files from the last run if any
	 rm -f /tmp/${job}.connect_to_${dbname}* > /dev/null
else
     leave 1 "$0 : This program is only designed to run from root. Sorry."
fi

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${dbname}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | grep $dbname | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# ... and reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
touch $lockfile

cat > /tmp/${job}.connect_to_${dbname} << END
connect to ${dbname} ;
connect reset ;
END

# disable trap either side of return code capture
trap " " ERR
su - ${instance} -c "db2 -tvf /tmp/${job}.connect_to_${dbname} > /tmp/${job}.connect_to_${dbname}.out"
rc=$?
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

(( rc == 0 )) &&  leave 0 "$0 : database $dbname is connectable at instance ${instance}."
(( rc > 0 ))  &&  leave 2 "$0 : Could not connect to database $dbname at instance ${instance}; please review /tmp/${job}.connect_to_${dbname}.out"
