#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins. 
# Script:		DB2_Database_Test4.ksh
# Arguments:	$1 instance name
#				$2 database alias [NB alias, not base name]
# Synopsis:		SQL replication is active and  error-free	return code 0
#				$2 is not registered for SQL replication   	return code 1
#				Replication has failed in the current hour 	return code 2
#				Any other error     						return code 3
#
# Instructions: #4 of several very basic DB2 heartbeat monitors for non-DBA staff monitoring through Nagios.
#				Do NOT run if an earlier test has failed - we assume at this stage that the instance is up and the database is activated and connectable.
# 				Run through root only at each database server in scope.
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Database_Test4.ksh instancename dbname
#
# Maintenance:
# 1. This is not a replacement for proper monitoring, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. Code contains a semaphore check in the form of a lock file to ensure Nagios doesn't run two or more instances of this script in parallel.
# 	 The semaphore check will exit immediately if a lock file is in place and another instance of the same job is running. 
#	 Do not remove the lockfile under this condition.
#
# Alex Levy, t/a http://www.sustainablesoftware.net October 2012
#
###########################################

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 -ne 4 ]] 
then rm -f $lockfile
fi
exit $RC
}
# Main
set -u        # assert undefined variables are not to be used
typeset here=$(hostname)
typeset timestamp=$(date +%Y-%m-%d-%H) # timestamp string YYYYY-MM-DD-HH to search in replication logs
typeset job=$(basename $0 .ksh)
typeset me=$(whoami)
typeset -i rc=0		# *nix return code 
typeset -i count=0	# counter
typeset lockfile=defined_further_down
typeset -i RC 		# set in function leave
typeset -i jobcount=0

# Check root is running this - some commands below require either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then :  
else
     leave 1 "$0 : This program is only designed to run from root. Sorry."
fi
	
(( $# == 2 )) || leave 3 "$job expects 2 arguments but received $# - usage $0 DB2instancename databasename."
typeset -l instance=$1
typeset -u dbname=$2

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${dbname}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | grep $dbname | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# ... and reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
touch $lockfile

# remove files from the last run if any; name pattern must not match lockfile.
rm -f /tmp/${job}.*.${dbname}* > /dev/null

# Generate a script for the instance owner
cat > /tmp/${job}.replication_checker.${dbname} << END
#!/bin/ksh
job=$job
typeset -i count=0 errcount=0
typeset -L logfile
. ~${instance}/sqllib/db2profile
# Check that the instance sqlllib/userprofile can identify whether the database is a replication capture server, apply server or neither:
[[ -n \$SQL_REPLICATION_SOURCE && -n \$SQL_REPLICATION_TARGET ]] || return 1
# signal if this database is not registered for either SQL Replication apply or capture
(( SQL_REPLICATION_SOURCE == 0 && SQL_REPLICATION_TARGET == 0 )) && return 2
if   (( SQL_REPLICATION_SOURCE == 1 ))
then count=\$(ps -eaf | grep $instance | grep -v grep | grep -c asncap)
	 (( count == 0 )) && return 3
	 # asncap capture_server=EM3PRCOR capture_schema=ASN capture_path=/database/replication/capture_files startmode=warmsi
	 capture_server=\$(ps -eaf | grep asn | grep -v grep | awk '{print \$9}')
	 capture_server=\${capture_server#capture_server=}
	 capture_schema=\$(ps -eaf | grep asn | grep -v grep | awk '{print \$10}') 
	 capture_schema=\${capture_schema#capture_schema=}
	 capture_path=\$(ps -eaf | grep asn | grep -v grep | awk '{print \$11}')
	 capture_path=\${capture_path#capture_path=}
	 #  default name pattern
	 logfile=\${capture_path}/${instance}.\${capture_server}.\${capture_schema}.CAP.log
     errcount=\$(grep $timestamp \$logfile | grep -c ASN[0-9][0-9][0-9][0-9]E )
 	 (( errcount > 0 )) && return 4
	 return 5     
fi
if   (( SQL_REPLICATION_TARGET == 1 ))
then count=\$(ps -eaf | grep $instance | grep -v grep | grep -c asnapply)
	 (( count == 0 )) && return 6
	 apply_qual=\$(ps -eaf | grep asn | grep -v grep | awk '{print \$9}')
	 apply_qual=\${apply_qual#apply_qual=}
 	 control_server=\$(ps -eaf | grep asn | grep -v grep | awk '{print \$10}')
	 control_server=\${control_server#control_server=}
	 apply_path=\$(ps -eaf | grep asn | grep -v grep | awk '{print \$11}')
	 apply_path=\${apply_path#apply_path=}
	 #  default name pattern instance.<control_server>.<apply_qual>.APP.log
	 logfile=\${apply_path}/${instance}.\${control_server}.\${apply_qual}.APP.log
     errcount=\$(grep $timestamp \$logfile | grep -c ASN[0-9][0-9][0-9][0-9]E )
 	 (( errcount > 0 )) && return 7
     return 8     
fi
return 99 # catch anything that falls through
END

chmod +x /tmp/${job}.replication_checker.${dbname}
trap " " ERR # turn trap off from here on as we want to evaluate return codes
su - $instance -c /tmp/${job}.replication_checker.${dbname} > /tmp/${job}.replication_checker.${dbname}.out # redirect any output from execution of .profile, e.g. 'you have mail' messages.
rc=$? 
case $rc in
	 1) leave 1 "$0 : Incomplete userprofile - see setup notes in the README." ;;
	 2) leave 1 "$0 : Deschedule this job at this server! SQL Replication is not configured here." ;;
	 3) leave 2 "$0 : Capture is not running at instance $instance database $dbname and is presumed down." ;;
	 4) leave 2 "$0 : Please review errors in the capture log  within the last hour of YYYY-MM-DD-HH ${timestamp} for instance $instance database ${dbname}." ;;
	 5) leave 0 "$0 : SQL Replication capture appears to be running normally at instance $instance database ${dbname}." ;;
	 6) leave 2 "$0 : Apply is not running at instance $instance database $dbname and is presumed down." ;;
	 7) leave 2 "$0 : Please review errors in the apply log  within the last hour of YYYY-MM-DD-HH ${timestamp} for instance $instance database ${dbname}." ;;
	 8) leave 0 "$0 : SQL Replication apply appears to be running normally at instance $instance database ${dbname}." ;;
	 *) # WTF
		leave 3 "$0 : Undefined error, sorry. Please review, correct and rerun /tmp/${job}.replication_checker.${dbname}" ;;
esac

