#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins.
# Script:		DB2_Database_Test5.ksh
# Arguments:	$1 instance name
#				$2 database alias 
# Synopsis:		How old is the most recent backup? Are we meeting the backup schedule?
#				As supplied, this script assumes once-daily backups, and therefore tests for a recent backup initiated within the last 27 hours.
#				Adjust according to your local site needs.
#				There is a recent full backup 				return code 0
#				Latest backup > 27 hours old				return code 1
#				There are no backups at all!               	return code 2
#				Any other error     						return code 3
#
# Instructions: #5 of several very basic DB2 heartbeat monitors for non-DBA staff monitoring through Nagios.
#				Do NOT run if an earlier test has failed - we assume at this stage that the instance is up and the database is activated and connectable.
# 				Run through root only at each database server in scope.
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Database_Test4.ksh instancename dbname
#
# Maintenance:
# 1. This is not a replacement for proper monitoring, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. Code contains a semaphore check in the form of a lock file to ensure Nagios doesn't run two or more instances of this script in parallel.
# 	 The semaphore check will exit immediately if a lock file is in place and another instance of the same job is running. 
#	 Do not remove the lockfile under this condition.
#
# Alex Levy, t/a http://www.sustainablesoftware.net October 2012
#
###########################################

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 -ne 4 ]] 
then rm -f $lockfile
fi
exit $RC
}

# Main
set -u        # assert undefined variables are not to be used
typeset here=$(hostname)
typeset job=$(basename $0 .ksh)
typeset me=$(whoami)
typeset -i rc=0		# *nix return code 
typeset lockfile=defined_further_down
typeset -i RC 		# set in function leave
typeset -i jobcount=0

# Check root is running this - some commands below require either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then : 
else
     leave 1 "$0 : This program is only designed to run from root. Sorry."
fi

(( $# == 2 )) || leave 3 "$job expects 2 arguments but received $# - usage $0 DB2instancename databasename."
typeset -l instance=$1
typeset -u dbname=$2

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${dbname}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | grep $dbname | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# ... and reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
touch $lockfile

# remove files from the last run if any, but not the lockfile.
rm -f /tmp/${job}.backup*${dbname} > /dev/null

# Generate a script for the instance owner
cat > /tmp/${job}.backup_checker.${dbname} << END
#!/bin/ksh
typeset last_backup=0
typeset now_minus_27hours=0
typeset -i rc=0

. ~${instance}/sqllib/db2profile
db2 -v connect to $dbname || return 1

now_minus_27hours=\$(db2 -x "values substr(replace(replace(char(current timestamp - 27 hours),'-',''),'.',''),1,14)")
rc=$? ; (( rc > 0 )) && return 2

# operation = 'B' = backup
# operationtype = 'F' offline or 'N' online - we're not inmterested in incremental/delta
# objecttype = 'D' = full database
# sqlcode is null  -- bit odd this, it's only populated on failure
# sqlwarn is null -- ditto
# start_time is a varchar(14) representation of a timestamp YYYYMMDDhhmmss
last_backup=\$(db2 -x "select coalesce(max(start_time), 0)  \
                      from   sysibmadm.db_history           \
                      where  operation = 'B'                \
                      and    objecttype = 'D'               \
                      and    operationtype in ('F','N')     \
                      and    sqlcode is null                \
                      and    sqlwarn is null                \
                      and    location <> '/dev/null' ")
rc=$?
db2 -v terminate
print Now minus 27 hours: \$now_minus_27hours
print Timestamp for last successful full backup is \$last_backup
(( rc > 0 )) && return 3

(( last_backup >= now_minus_27hours )) && return 0
(( last_backup == 0 )) && return 4
(( last_backup < now_minus_27hours ))  && return 5

return 99 # catch anything that falls through
END

chmod +x /tmp/${job}.backup_checker.${dbname}
trap " " ERR # turn trap off as we want to evaluate return codes
su - $instance -c /tmp/${job}.backup_checker.${dbname} > /tmp/${job}.backup_checker.${dbname}.out
rc=$? 
# reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
case $rc in
	 0) leave 0 "$0 : There is a good, recent full backup image for db ${dbname}. See /tmp/${job}.backup_checker.${dbname}.out for details." ;;
	 1) leave 3 "$0 : Connection failure to db ${dbname}. See /tmp/${job}.backup_checker.out for details." ;;
	 2) leave 3 "$0 : SQL query error (query 1) - please review /tmp/${job}.backup_checker.out." ;;
	 3) leave 3 "$0 : SQL query error (query 2) - please review /tmp/${job}.backup_checker.out." ;;
	 4) leave 2 "$0 : There has NEVER been a successful backup for database ${dbname} !!! Recommend immediate backup." ;;
	 5) leave 2 "$0 : The last good full backup for database ${dbname} is out of date. Recommend immediate backup." ;;
	 *) # WTF
		leave 3 "$0 : Undefined error, sorry. Please review, correct and rerun /tmp/${job}.backup_checker.${dbname}" ;;
esac
