#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins. 
# Script:		DB2_Database_Test6.ksh
# Arguments:	$1 instance name
#				$2 database alias 
# Synopsis:		Test that archive logging is working correctly.
#				The only obvious and simple way I can think of to check transaction logs are being archived is to run an archive log command and check the db history.
#				Since the archive log command is asynchronous and Nagios has a default timeout of 30 seconds, we cannot wait for the archive itself to complete.
#				Therefore simply check that at least 1 log has been archived during the last 2 hours, or whatever the run frequency you set for this plugin.
#				Archive logging working       				return code 0
#				Database not configured for archive logging return code 0
#				Log directory > 90% full     				return code 2
# 				Missing archive log directory ($archlogdir) return code 2
#				Archive logging does not work              	return code 2
#				Any other error     						return code 3
#
# Instructions: #6 of several very basic DB2 heartbeat monitors for non-DBA staff monitoring through Nagios.
#				Edit the full path to the archive log directory in variable $archlogdir. This is not visible in the db cfg if you are using vendor software.
#				Do NOT run if an earlier test has failed - we assume at this stage that the instance is up and the database is activated and connectable.
# 				Run through root only at each database server in scope.
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Database_Test6.ksh instancename dbname
#
# Maintenance:
# 1. This is not a replacement for proper monitoring, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. Code contains a semaphore check in the form of a lock file to ensure Nagios doesn't run two or more instances of this script in parallel.
# 	 The semaphore check will exit immediately if a lock file is in place and another instance of the same job is running. 
#	 Do not remove the lockfile under this condition.
#
# Alex Levy, t/a http://www.sustainablesoftware.net October 2012
#
###########################################

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 -ne 4 ]] 
then rm -f $lockfile
fi
exit $RC
}

# Main
set -u        # assert undefined variables are not to be used
typeset here=$(hostname)
typeset job=$(basename $0 .ksh)
typeset me=$(whoami)
typeset archlogdir= #< you fill this in> # <====         
typeset archlogdir_utilisation_pct	#n%
typeset -i archlogdir_utilisation=0 #n with the percent sign stripped off
typeset -i rc=0		# *nix return code 
typeset lockfile=defined_further_down
typeset -i RC 		# set in function leave
typeset -i jobcount=0

# Check root is running this - some commands below require either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then : 
else 
     leave 1 "$0 : This program is only designed to run from root. Sorry."
fi

(( $# == 2 )) || leave 3 "$job expects 2 arguments but received $# - usage $0 DB2instancename databasename."
typeset -l instance=$1
typeset -u dbname=$2

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${dbname}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | grep $dbname | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# ... and reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
touch $lockfile
# remove files from the last run if any, but leave the lockfile.
rm -f /tmp/${job}.archlog_checker.${dbname}* > /dev/null

# Verify that the archive log directory exists: 
if   [[ -d ${archlogdir} ]] 
then :
else leave 2 "$0 : archive log directory ${archlogdir} is not present."
fi

# Test ${archlogdir} utilisation is below 90% of filesystem capacity. A df -h ${archlogdir} produces output like:
# Filesystem            Size  Used Avail Use% Mounted on
# /db2home/db2inst1     131G   40G   84G  33% /db2home
if  [[ -d  ${archlogdir} ]]
then 
	archlogdir_utilisation_pct=$(df -h ${archlogdir} | egrep "/" | tail -1 | awk '{print $5'}) 
	archlogdir_utilisation=${archlog_utilisation_pct%\%}
   (( archlogdir_utilisation > 90 )) &&  leave 2 "$0 : archive log directory is ${archlogdir_utilisation_pct} full." 
fi

# Generate a script for the instance owner
cat > /tmp/${job}.archlog_checker.${dbname} << END
#!/bin/ksh
typeset  last_archive=0 
typeset  now_minus_2hours=0 
typeset -i rc=0

. ~${instance}/sqllib/db2profile
db2 -v archive log for database $dbname
rc=\$? ; (( rc > 0 )) && return 3
# The archive log command establishes its own connection, so we need to connect for remaining queries:
db2 -v connect to $dbname || return 1
#now_minus_2hours=\$(db2 -x "values substr(replace(replace(char(current timestamp - 2 hours),'-',''),'.',''),1,14)")
#this is smarter:
now_minus_2hours=\$(db2 -x "values to_char(current timestamp - 2 hours,'YYYYMMDDHH24MISS')")
rc=\$? ; (( rc > 0 )) && return 2

# The archive log command is asynchronous, or at any rate not instantaneous.
# Therefore query the database history to see if there is any record of a successful archive in the last 2 hours.
#sleep 60 <=== commented out; Nagios timeout interval is 30 seconds
# operation = 'X' = archive log
# operationtype = 'N' forced via archive log ommand or 'P' primary or '1' logarchmeth1
# sqlcode is null  -- bit odd this, it's only populated on failure
# sqlwarn is null -- ditto
# start_time is a varchar(14) representation of a timestamp YYYYMMDDhhmmss
last_archive=\$(db2 -x "select coalesce(max(start_time), 0)	\
                      from   sysibmadm.db_history           \
                      where  operation = 'X'                \
                      and    operationtype in ('N','P','1')	\
					  and    sqlcode is null                \
                      and    sqlwarn is null ")
rc=\$? 
db2 -v terminate
print Now minus 2 hours: \$now_minus_2hours
print Timestamp for last successful transaction log archive is \$last_archive 
(( rc > 0 )) && return 4

(( last_archive >= now_minus_2hours )) && return 0
(( last_archive == 0 )) && return 5
(( last_archive < now_minus_2hours ))  && return 6

return 99 # catch anything that falls through
END

chmod +x /tmp/${job}.archlog_checker.${dbname}
trap " " ERR # turn trap off temporarily as we want to evaluate return codes
su - $instance -c /tmp/${job}.archlog_checker.${dbname} > /tmp/${job}.archlog_checker.${dbname}.out
rc=$? 
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
case $rc in
	 0) leave 0 "$0 : There is a good, recent full archive image for db ${dbname}. See /tmp/${job}.archlog_checker.out for details." ;;
	 1) leave 3 "$0 : Connection failure to db ${dbname}. See /tmp/${job}.archlog_checker.out for details." ;;
	 2) leave 3 "$0 : SQL query error (query 1) - please review /tmp/${job}.archlog_checker.out." ;;
	 3) leave 2 "$0 : Forced archive log error - please review /tmp/${job}.archlog_checker.out." ;;
	 4) leave 3 "$0 : SQL query error (query 2) - please review /tmp/${job}.archlog_checker.out." ;;
	 5) leave 2 "$0 : There is no history of successful transaction log archiving at database $dbname - please review /tmp/${job}.archlog_checker.out." ;;
	 6) leave 1 "$0 : The last successful archive log action for database ${dbname} was more than an hour ago. Check if this persists before calling DBA Support." ;;
	 *) # WTF
		leave 3 "$0 : Undefined error, sorry. Please review, correct and rerun /tmp/${job}.archive_checker.${dbname}" ;;
esac
