#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins, Confluence page: https://repo.dev.bbc.co.uk/dmi/em3/dev/trunk/EM3_DB/core/deploy_db/bin/NAGIOS/ 
# Script:		DB2_Database_Test8.ksh
# Arguments:	$1 instance name
#				$2 database alias 
# Synopsis:		Test that sequence objects are within acceptable thresholds of their maximum values.
# 				This applies to EM3 Core only - FABOPS-9421 refers.
#				
#				All sequences within safety limits			return code 0
#				Database out of scope for this test         return code 1
#				At least 1 sequence > 80% used				return code 1
#				At least 1 sequence > 90% used				return code 2
#				Any other error     						return code 3
#
# Instructions: #8 of several very basic DB2 heartbeat monitors for non-DBA staff monitoring through Nagios.
#				Do NOT run if an earlier test has failed - we assume at this stage that the instance is up and the database is activated and connectable.
# 				Run through root only at each database server in scope - EM3 Core only, not replica/GUI, not reporting.
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Database_Test8.ksh instancename dbname
#
# DB2 Notes:	1. It would have been much simpler to compare the next value for the sequence against its maxvalue, however even with autocommit off
#				it's impossible to get the next value without incrementing the sequence itself; that would be self-defeating.             
#a				2. Nagios constrains the script to 1 line of output, therefore we cannot safely serialise a list of sequence names.
#
# Maintenance:
# 1. This is not a replacement for proper monitoring, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. A list of DB2 instance and database names by server is at:
#    https://confluence.dev.bbc.co.uk/display/DMI/EM3+DB2+Database+inventory?showChildren=true#children
# 
# Alex Levy, 20.08.2012 Initial version
#
# Amendment History
# -----------------
# Alex, 12.09.2012	Add semaphore check to ensure Nagios doesn't run two or more instances of this script in parallel.
# Alex, 29.10.2012	FABOPS-11414/8: Alter semaphore check so that we exit immediately if a lock file is in place and another instance of the same job is running. 
#					Do not remove the lockfile under this condition.
# 					FABOPS-7491. Recode to workround Nagios timeout limit of 30 seconds. The problem here is that it takes longer than 30 seconds to index-scan 300+ tables.
#                   A runtime > 30 seconds generates a Nagios NRPE:Service timed out critical error and the timeout cannot be adjusted for individual checks.
#                   Since this job runs every 2 hours, we will run the checks in background mode and test the output from the previous run. Yes, I know.
# Alex, 25.03.2013	FABOPS-9425: This job was descheduled, but not known when. Press back into service. Correct population of .severe and .warning files.

#
###########################################

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 -ne 4 ]] 
then rm -f $lockfile
fi
exit $RC
}

# Main
set -u        		# assert variables must be defined
typeset job=$(basename $0 .ksh)
typeset -i count=0	# counter for leave function
typeset here=$(hostname)
typeset me=$(whoami)
(( $# == 2 )) || leave 3 "$job expects 2 arguments but received $# - usage $0 DB2instancename databasename."
typeset -l instance=$1
typeset -u dbname=$2
typeset -i rc=0		# *nix return code 
typeset lockfile=defined_further_down
typeset -i RC 				# set in function leave
typeset -i jobcount=0

# Check root is running this - some commands below require either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then : 
else
     leave 1 "$0 : This program is only designed to run from root. Sorry."
fi

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${dbname}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | grep $dbname | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# ... and reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
touch $lockfile

# Generate a script for the instance owner
cat > /tmp/${job}.sequence_checker.${dbname} << END
#!/bin/ksh

typeset -i severe=0 warning=0 # threshold counters 
typeset -i rc=0 rc1=0 
. ~${instance}/sqllib/db2profile

# only proceed to the next DB2 operation if the previous one succeeds; accumulate the return code at each step.
db2 -v "connect to $dbname" 
rc=\$?

if (( rc == 0 ))
then
db2 -v "declare global temporary table whatseq (	\
		tabschema   varchar(128) not null,			\
		seqname     varchar(128) not null,			\
		maxseq      bigint       not null,			\
		tabname     varchar(128) ,					\
		colname		varchar(128) ,					\
		maxval      bigint       					\
		) in utmp4k									\
		on commit preserve rows"
rc1=\$? 
fi
(( rc += rc1 ))

if (( rc == 0 ))
then
db2	-mv "insert into session.whatseq				\
		(tabschema, seqname, maxseq )				\
		select 	s.seqschema, s.seqname, s.maxvalue	\
		from 	syscat.sequences s inner join 		\
			 	syscat.datatypes d 					\
		on 		s.datatypeid = d.typeid				\
		where 	s.seqtype = 'S'						\
		and   	s.seqschema not like 'SYS%'			\
		and   	d.typename = 'SMALLINT'"			
rc1=\$?
fi
(( rc += rc1 ))

if (( rc == 0 ))
then
db2 -mv "update 	session.whatseq w				\
		set    	(tabname,colname) = 				\
				(select t.tabname, c.colname 		\
				from 	syscat.tables t inner join 	\
						syscat.columns c 			\
				on 		(t.tabschema,t.tabname) = (c.tabschema,c.tabname)	\
                where 	t.type = 'T' 										\
				and 	t.tabschema = w.tabschema 							\
				and 	t.tabname = replace(w.seqname,'_SEQ')  				\
                and 	c.colno = 0 										\
				and 	c.typename = 'SMALLINT' )"
rc1=\$?
fi
(( rc += rc1 ))

if (( rc == 0 ))
then
# the next statement generates the remaining update SQL
db2 -x "select 	'update session.whatseq',													\
       			' set maxval = (select coalesce(max(', substr(colname,1,40), '),0) from ',	\
				char(rtrim(tabschema)||'.'||tabname,60), 									\
				' ) where tabschema = ', ''''||tabschema||'''',								\
				' and tabname = ', ''''||tabname||'''',										\
				';' 																		\
		from   session.whatseq																\
		where  colname is not null" > /tmp/${job}.${dbname}.update.sql
rc1=\$?
fi
(( rc += rc1 ))

if   (( rc == 0 ))
then db2 -tvf /tmp/${job}.${dbname}.update.sql 
     rc1=\$? 
fi
(( rc += rc1 ))

if  (( rc == 0 ))
then 
    db2 -v "export to  /tmp/${job}.sequence_export.${dbname}.del of del select seqname, tabschema, tabname, colname, maxval, maxseq from session.whatseq"
	severe=\$(db2 -x "select count(*) from  session.whatseq where maxval > maxseq * 0.9 and colname is not null")
	rc1=\$? ;  (( rc > 1 )) && (( rc += rc1 ))
    warning=\$(db2 -x "select count(*) from  session.whatseq where maxval > maxseq * 0.8 and colname is not null")
	rc1=\$? ;  (( rc > 1 )) && (( rc += rc1 ))
fi

((severe > 0 )) && db2 -v "select * from session.whatseq where maxval > maxseq * 0.9 and colname is not null"
((warning > 0 )) && db2 -v "select * from session.whatseq where maxval > maxseq * 0.8 and colname is not null"

db2 -v terminate

# set final return code (see root's case statement below for interpretation ))
if   (( rc == 0 ))
then if   (( severe > 0 ))
     then rc=2
     else if (( warning > 0 ))
          then rc=1
          fi
     fi
else # signal a SQL error
     rc=4
fi
print \$rc      > /tmp/${job}.${dbname}.rc
print \$severe  > /tmp/${job}.${dbname}.severe
print \$warning > /tmp/${job}.${dbname}.warning
chmod 444 /tmp/${job}.${dbname}.rc /tmp/${job}.${dbname}.severe  /tmp/${job}.${dbname}.warning /tmp/${job}.${dbname}.update.sql /tmp/${job}.sequence_export.${dbname}.del
END

# Analyse results from previous run

# remove main output file from the last run, if any
rm -f /tmp/${job}.sequence_checker.${dbname}.out > /dev/null

chmod +x  /tmp/${job}.sequence_checker.${dbname} 
trap " " ERR # turn trap off temporarily as we want to evaluate the return code explicitly:
su - $instance -c "nohup /tmp/${job}.sequence_checker.${dbname} > /tmp/${job}.sequence_checker.${dbname}.out &" 
#rc=$? # no return code check, since we cannot wait for it to complete
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

if  [[ -f /tmp/${job}.${dbname}.rc ]]
then
rc=$(cat /tmp/${job}.${dbname}.rc)
case $rc in
	 0) leave 0 "$0 : All sequences are within acceptable limits at database ${dbname}." ;;
	 1) leave 1 "$0 : At least 1 sequence is > 80% utilised OR arguments are incorrect. Please review /tmp/${job}.sequence_export.${dbname}.del ." ;;
	 2) leave 2 "$0 : At least 1 sequence is > 90% utilised. Please review /tmp/${job}.sequence_export.${dbname}.del ." ;;
	 4) leave 3 "$0 : SQL query or connection error - please review /tmp/${job}.sequence_checker.${dbname}.out." ;;
	 *) # WTF
		leave 3 "$0 : Undefined error, sorry. Please review, correct and rerun /tmp/${job}.sequence_checker.${dbname}" ;;
esac
else # Assume this is the first run if there are no outputs from previous runs.
leave 0 "$0 : Presumed first run at this host."
fi
