#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins. 
# Script:		DB2_Instance_Test1.ksh
# Arguments:	$1 instance name
# Synopsis:		Is DB2 instance $1 started?  	return code 0
#				If stopped, is it startable? 	return code 1
#				Or is the instance inoperable? 	return code 2
#				Any other error     			return code 3
#
# Instructions: #1 of several very basic DB2 heartbeat monitors for non-DBA staff monitoring through Nagios.
# 				Run through root only at each database server in scope.
# 				Run for each DB2 instance at the server.
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Instance_Test1.ksh instancename
#
# Maintenance:
# 1. This is not a replacement for proper monitoring, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. Code contains a semaphore check in the form of a lock file to ensure Nagios doesn't run two or more instances of this script in parallel.
# 	 The semaphore check will exit immediately if a lock file is in place and another instance of the same job is running. 
#	 Do not remove the lockfile under this condition.
#
# Alex Levy, t/a http://www.sustainablesoftware.net October 2012
#
###########################################

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 -ne 4 ]] 
then rm -f $lockfile
fi
exit $RC
}

# Main

set -u        # assert undefined variables are not to be used
typeset here=$(hostname)
typeset timestamp=$(date +%Y%m%d%H%M)
typeset job=$(basename $0)
typeset db2_install_path=   # Linux default at DB2 9.7 is /opt/ibm/db2/V9.7, but will change at next version
typeset db2_cmd_path=
typeset -l instance_state=
typeset me=$(whoami)
typeset lockfile=defined_further_down
typeset -i RC 				# set in function leave
typeset -i jobcount=0

# Check root is running this - the db2gcf command requires either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then :
else
     leave 1 "$0 : This program is only designed to run from root. Sorry."
fi

(( $# == 1 )) || leave 3 "$job expects 1 argument but received $# - usage $0 DB2instancename."
typeset -l instance=$1

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${instance}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# ... and reset trap
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "$0 : Unknown: Logical or untrapped error"
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR
touch $lockfile

# Check if this is a DB2 server - is DB2 installed? and if so, what is the install path
db2_install_path=$( /usr/local/bin/db2ls | tail -1 | awk '{print $1}')
[[ -d ${db2_install_path} && -x  ${db2_install_path} ]] || leave 1 "$0 : DB2 is not installed here at ${here}. Nothing to check. Exiting."
# Check the db2gcf and db2ilist commands are executable
[[ -f ${db2_install_path}/bin/db2gcf && -x ${db2_install_path}/bin/db2gcf ]] || leave 2 "$0 : Cannot execute /opt/ibm/db2/V9.7/bin/db2gcf as ${me}. Check DB2 installation."
[[ -f ${db2_install_path}/instance/db2ilist && -x ${db2_install_path}/instance/db2ilist ]] || leave 2 "$0 : Cannot execute /opt/ibm/db2/V9.7/instance/db2ilist as ${me}.Check DB2 installation."
typeset -i i=$( ${db2_install_path}/instance/db2ilist | grep -c $instance ) 
(( i == 1 )) || leave 1 "$0 : $instance is not a DB2 instance at this server."

# Check if the instance has failed over; the tell-tale sign is if key directories such as sqllib are still under $INSTHOME, or have been remounted to the standby server.
# Further, test the path to DB2 commands must be executable.
db2_cmd_path=~${instance}/sqllib/bin
if   [[ -d ${db2_cmd_path} && -x ${db2_cmd_path} && -x ~${instance}/sqllib/db2profile ]]
then . ~${instance}/sqllib/db2profile
     instance_state=$(/opt/ibm/db2/V9.7/bin/db2gcf -s -i $instance | egrep "DB2 State" | awk -F": " '{print $2}')
     case $instance_state in
          "available") leave 0 "DB2 instance $instance is up and running at host ${here}." ;;
          "operable" ) leave 1 "DB2 instance $instance is down but can be brought up at host ${here}." ;;
          "not ope*" ) leave 2 "DB2 instance $instance is not operable and cannot be brought up at host ${here}." ;;
     esac
else
     leave 1 "$0 : It looks like DB2 instance $instance has failed over, please check."
fi
