#!/bin/ksh
# set tabstop=4 to display correctly in vi
#
# Suite:		Nagios DB2 plugins, Confluence page: https://repo.dev.bbc.co.uk/dmi/em3/dev/trunk/EM3_DB/core/deploy_db/bin/NAGIOS/ 
# Script:		DB2_Database_Test13.ksh
# Arguments:	$1 instance name
# Synopsis:		Health check for severe error messages in the DB2 diagnostic log in the last hour.
#				
#				No severe messages in the last hour.	   return code 0
#				1 or more severe messages in the last hour return code 2
#				Any other error     					   return code 3
#
# Instructions: #13 of several very basic DB2 monitors for non-DBA staff monitoring through Nagios.
#				Do NOT run if an earlier test has failed - we assume at this stage that the instance is up.
# 				Run through root only at each database server in scope. 
# 				Call from any daemon/wrapper with an appropriate repeat loop interval.
# Usage: 		<path>/DB2_Database_Test13.ksh instancename 
#
# DB2 Notes:	
#
# Maintenance:
# 1. This is not a replacement for a proper monitoring soluion like DBI tools, just an ad hoc and very basic stopgap.
# 2. Instructions at http://nagiosplug.sourceforge.net/developer-guidelines.html#DEVREQUIREMENTS constrain the scripts to 1 line of output only.
# 3. Code contains a semaphore check in the form of a lock file to ensure Nagios doesn't run two or more instances of this script in parallel.
# 	 The semaphore check will exit immediately if a lock file is in place and another instance of the same job is running. 
#	 Do not remove the lockfile under this condition.
#
# Alex Levy, t/a http://www.sustainablesoftware.net March 2013
#
# Amendment History
# -----------------
#
#
###########################################

# Capture any unidentified error conditions 
# - we don't know what the error is, but we do know there is one.
trap 'RC=${RC:-999}  # The leave function sets this variable; assign a value of 999 if the leave function was not invoked.
     (( RC == 999 )) && print "Critical: Unknown: Logical or untrapped error."
     (( RC == 999 )) && RC=3
	 if   [[ -f $lockfile && $jobcount -eq 1 ]]
	 then rm -f $lockfile
	 fi
	 exit $RC  
     ' ERR

function leave {
# Ends the script with message $2 and caller-supplied exit-code $1.
# The exit at the end of this function will resignal the trap, and that's what will actually exit. 

(( $# != 2 )) && print "Unknown: function leave in $job expects 2 arguments but got $# " && exit 3
RC=$1
case $1 in
   	 0) print "OK: ${2}" 	   
		;;	
   	 1) print "Warning: ${2}"  
		;;
   	 2) print "Critical: ${2}" 
		;;
   	 3) print "Unknown: ${2}"  
		;;   
   	 4) print "Critical: ${2}"  # DUPLICATE RUN
		RC=3
        ;;   
	 *) print "Unknown: Logical or untrapped error"
	    RC=3 
	    ;;   
esac

if   [[ -f $lockfile && $1 != 4 ]] 
then rm -f $lockfile
fi
exit $RC
}

# Main
set -u        		# assert variables must be defined
typeset job=$(basename $0 .ksh)
typeset -i count=0	# counter for leave function
typeset here=$(hostname)
typeset me=$(whoami)
(( $# == 1 )) || leave 3 "$job expects 1 argument but received $# - usage $0 DB2instancename."
typeset -l instance=$1
typeset -i rc=0		# *nix return code 
typeset lockfile=defined_further_down
typeset -i RC 		# set in function leave
typeset -i jobcount=0

# Check root is running this - some commands below require either root or DB2 sysadm authority:
if   [[ $me == "root" ]]
then : 
else
     leave 1 "$0 : This program is only designed to run from root. Sorry."
fi

# Check this job is not already running; prevent duplicates from running concurrently.
lockfile=/tmp/${job}.${instance}.lock
if   [[ -f $lockfile ]]
then 
	 trap " " ERR # disable trap since grep can legitimately return non-zero
	 jobcount=$(ps -eaf | grep $job | grep $instance | grep $dbname | egrep -c "/bin/ksh" )
	 (( jobcount > 1 )) && leave 4 "Another run of this job is already in progress, so this one ends. Please investigate."
	 (( jobcount == 1 )) && rm -f $lockfile # only this instance of the job is running
	 (( jobcount == 0 )) && rm -f $lockfile # earlier instance(s) failed but left lockfile in place
fi
# leave trap unset: the db2diag command in the next script returns 1 if no errors found, 0 if found!!!

# Generate a script for the instance owner:
# This will exit with 0 (no severe errors found), or the latest line number in the diagnostic log with a severe message.
cat > /tmp/${job}.${instance} << END
#!/bin/ksh
typeset -i severe_lines=0 last_severe_line=0
. ~${instance}/sqllib/db2profile
severe_lines=$( ~${instance}/sqllib/bin/db2diag -H 1h -level Severe | wc -l)
(( severe_lines > 0 )) && last_severe_line=$(grep -n Severe ~${instance}/sqllib/db2dump/db2diag.log | tail -1 | awk -F":" '{print $1}')
exit \$last_severe_line
END

chmod +x  /tmp/${job}.${instance} 
su - $instance -c "/tmp/${job}.${instance} > /tmp/${job}.${instance}.out " 
rc=$? 
if   (( rc == 0 ))
then print "$0 : No severe errors have been reported in the $instance DB2 diagnostic log within the past hour. Phew."
	 exit 0
else print "$0 : $rc severe errors have been reported in the $instance DB2 diagnostic log within the past hour."
	 exit 2
fi
